part of dom_node;

/**
 * This class allows operating with arbitrary node data.
 */
abstract class DomNodeDataCapable {
  /**
   * Data namespace.
   */
  static const _NAMESPACE = '__dartQuery__';
  
  /**
   * Gets the internal elements.
   */
  List<Element> get elements;
  
  /**
   * Gets or sets data.
   */
  Object data(String name, [Object value]) {
    if (value != null) {
      _setData(name, value);
    }
    
    return _getData(name);
  }
  
  /**
   * Gets the data object of an element.
   */
  Map _getDataObject(Element element) {
    var attr = element.getAttribute(_NAMESPACE);
    var data = attr != null? JSON.decode(attr): {};
    
    if (data is! Map) {
      throw new exception.DomException('The node doesn\'t contain valid data');
    }
    
    return data;
  }
  
  /**
   * Sets the data object of an element.
   */
  void _setDataObject(Element element, Map data) {
    element.setAttribute(_NAMESPACE, JSON.encode(data));
  }
  
  /**
   * Gets data.
   */
  Object _getData(String name) {
    var str = null;
    
    if (elements.length > 0) {
      var data = _getDataObject(elements[0]);
      
      if (data.containsKey(name)) {
        str = data[name];
      }
    }
    
    return str;
  }
  
  /**
   * Sets data.
   */
  void _setData(String name, Object value) {
    elements.forEach((element) {
      var data = _getDataObject(element);
      
      data[name] = value;
      _setDataObject(element, data);
    });
  }
}